Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type Generator ¶
type Generator struct {
AgentName string
ModulePath string
CanisterName string
PackageName string
ServiceDescription did.Description
// contains filtered or unexported fields
}
Generator is a generator for a given service description.
func NewGenerator ¶
NewGenerator creates a new generator for the given service description.
func (*Generator) GenerateMock ¶
Click to show internal directories.
Click to hide internal directories.