Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type HandlerFunc ¶
type Method ¶
type Method struct {
Name string // Name is the name of the method.
Arguments []any // Arguments is a list of pointers to the arguments, that will be filled by the agent.
Handler HandlerFunc // Handler is the function that will be called when the method is called.
}
Click to show internal directories.
Click to hide internal directories.